/*
 * @(#)InstancesQuery.java	1.8 98/10/08
 *
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package hat.server;

import hat.model.*;

/**
 *
 * @version     1.2, 11/13/97
 * @author      Bill Foote
 */


class InstancesQuery extends QueryHandler {

    private boolean includeSubclasses;
    private boolean newObjects;

    public InstancesQuery(boolean includeSubclasses) {
	this.includeSubclasses = includeSubclasses;
    }

    public InstancesQuery(boolean includeSubclasses, boolean newObjects) {
	this.includeSubclasses = includeSubclasses;
	this.newObjects = newObjects;
    }

    public void run() {
	JavaClass clazz = snapshot.findClass(query);
	String instancesOf;
	if (newObjects)
	    instancesOf = "New instances of ";
	else
	    instancesOf = "Instances of ";
	if (includeSubclasses) {
	    startHtml(instancesOf + query + " (including subclasses)");
	} else {
	    startHtml(instancesOf + query);
	}
	if (clazz == null) {
	    error("Class not found");
	} else {
	    out.print("<strong>");
	    printClass(clazz);
	    out.print("</strong><br><br>");
	    JavaHeapObject[] objects = clazz.getInstances(includeSubclasses);
	    long totalSize = 0;
	    long instances = 0;
	    for (int i = 0; i < objects.length; i++) {
		if (newObjects && !objects[i].isNew())
		    continue;
		printThing(objects[i]);
		out.println("<br>");
		totalSize += objects[i].getSize();
		instances++;
	    }
	    out.println("<h2>Total of " + instances + " instances occupying " + totalSize + " bytes.</h2>");
	}
	endHtml();
    }
}
